#ifndef AIKA_H_
#define AIKA_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <core/Namespace-core.h>
#include <io/Namespace-io.h>
#include <lang/Namespace-lang.h>
#include <math/Namespace-math.h>
#include <render/Namespace-render.h>
#include <scene/Namespace-scene.h>
#include <img/Namespace-img.h>

/** 
 * @mainpage Aika API Documentation
 * @section author Author 
 * - Erick Jung (erickjung@gmail.com)
 *
 * @section intro Introduction
 *
 * Welcome to the Aika API documentation.\n\n
 * Aika is a simple and powerful Game Engine, written in C++ (without any platform dependence),\n
 * you could compile and run aika on several types of platform (including nintendo ds, symbian, etc).
 *
 * The main proposal of the engine is to create a portable and easy of use framework to create multimedia content, e.g. games.  
 *
 * @section license License
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 * @page compile Compile Instructions
 *
 * @section intro Details
 *
 * You could compile Aika using the visual c++ 8 project.
 *
 * - To use the Visual C++ 8, go to "\projects\vc8\".
 *
 * Pre-Processor:
 * AK_DYN_EXPORT
 * - Used to compile the lib/dll
 *
 * AK_DEBUG
 * - If you wanna use memory, assertion and trace traking
 *
 * AK_OPENGL
 * - Used to compile the engine with opengl support.
 *
 * AK_DIRECTX
 * - Used to compile the engine with directx support. (removed for now).
 */

/**
 * @brief
 * The aika namespace
 *
 * @note
 * Used to group all the namespaces
 */
namespace aika
{
    using namespace core;
    using namespace io;
    using namespace lang;
    using namespace math;
    using namespace render;
    using namespace scene;
    using namespace img;
}

#endif /*AIKA_H_*/
