#ifndef __DEVICE_CONFIG_H__
#define __DEVICE_CONFIG_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <render/Render.h>

namespace core
{
    /**
     * The render type.
     */
    enum E_DEVICE_TYPE
    {
        DT_WIN32 = 0,  //!< Windows type
        DT_LINUX	   //!< Linux type
    };

    /**
     * @brief
     * The Device configuration.
     */
    class DeviceConfig
    {
    public:
        /**
        * Constructor.
        */
        DeviceConfig() 
			: pTitleName(NULL), 
			pDataPath(NULL),
			width(0), 
			height(0),
			bits(32),
			renderType(render::RT_OPENGL),
			deviceType(DT_WIN32),
			useShaders(false),
			fullScreen(false)
		{}

        /**
        * The title name.
        */
        const char* pTitleName;

        /**
        * The data path.
        */
        const char* pDataPath;

        /**
        * The screen width.
        */
        int16 width;

        /**
        * The screen height.
        */
        int16 height;

        /**
        * The bits.
        */
        int16 bits;

		/**
        * The render type.
        */
        render::E_RENDER_TYPE renderType;

        /**
        * The device type.
        */
        core::E_DEVICE_TYPE deviceType;

        /**
        * Use shaders.
        */
        bool useShaders;

        /**
        * Use fullscreen.
        */
		bool fullScreen;
    };
}

#endif //__DEVICE_CONFIG_H__
