#ifdef WIN32

#ifndef DEVICEWIN32_H_
#define DEVICEWIN32_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <windows.h>
#include <core/Device.h>

namespace core
{
    /**
     * @brief
     * The windows device specific implementation.
     */
    class API_STATE DeviceWin32 : public Device
    {
    public:
        /**
         * Constructor
         */
        DeviceWin32();

        /**
         * Destructor
         */
        virtual ~DeviceWin32();

        /**
         * @see Device::run()
         */
        bool run();

        /**
         * @see Device::createWindow(render::E_RENDER_TYPE renderType)
         */
        bool createWindow(DeviceConfig &device);

		/**
         * Return the time in seconds
         */
		float32 getTimeInSeconds();

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: DeviceWin32";}

    private:
        /**
         * The window handle.
         */
        HWND _hWnd;

        /**
         * The application instance.
         */
        HINSTANCE _hInstance;

    };
}

#endif /*DEVICEWIN32_H_*/

#endif // WIN32
