#ifndef __INPUTMAP_H__
#define __INPUTMAP_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>

namespace core
{
    /**
     * @brief
	 * This is the input map class.
     */
	class API_STATE InputMap /*: public Object*/
	{
	public:
        /**
         * The keys
         */
		enum E_KEY_TYPE
		{
			KT_NONE,
			KT_A,
			KT_B,
			KT_C,
			KT_D,
			KT_E,
			KT_F,
			KT_G,
			KT_H,
			KT_I,
			KT_J,
			KT_K,
			KT_L,
			KT_M,
			KT_N,
			KT_O,
			KT_P,
			KT_Q,
			KT_R,
			KT_S,
			KT_T,
			KT_U,
			KT_V,
			KT_W,
			KT_X,
			KT_Y,
			KT_Z,
			KT_TAB,
			KT_SHIFT,
			KT_CTRL,
			KT_SPACE,
			KT_RETURN,
			KT_ESC,
			KT_LEFT,
			KT_RIGHT,
			KT_UP,
			KT_DOWN,
			KT_F1,
			KT_F2,
			KT_F3,
			KT_F4,
			KT_F5,
			KT_F6,
			KT_F7,
			KT_F8,
			KT_F9,
			KT_F10,
			KT_F11,
			KT_F12,
			KT_QUANTITY
		};

        /**
         * Create the input map instance.
         */
		static void createInstance();

        /**
         * Delete the input map instance.
         */
		static void deleteInstance();

		/**
         * Get the input map instance.
         * 
         * @return Returns the instance.
         */
		static InputMap *getInstance();

        /**
         * Receive the input key from the specific device.
		 * @note This only need to be used directly on the device.
         * 
         * @param key The input key.
         */
		static void inputKey(uint32 key);

        /**
         * Verify if the specific key was pressed.
         * 
         * @param eKey The key type.
         * @return Returns true if was pressed.
         */
		static bool verifyIfWasPressed(E_KEY_TYPE eKey);

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: InputMap";}

	private:
        /**
         * Use to verify if the key was pressed.
         */
		static bool _keyPressed;

        /**
         * The key type.
         */
		static E_KEY_TYPE _key;

        /**
         * The input key instance.
         */
		static InputMap *_instance;

        /**
         * Constructor.
         */
		InputMap(){}

		/**
         * Destructor.
         */
		~InputMap(){}
	};
}

#endif
