#ifndef __IMAGE_H__
#define __IMAGE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>
#include <lang/String.h>

namespace img
{
	using namespace lang;

	/**
     * @brief
     * This is the image class, responsible to load all the image files.
     * 
     * @note
     * For now we only have support for OpenIL.
     */
	class API_STATE Image : public Object
	{
	public:

		/**
         * Constructor
         */
		Image();

        /**
         * Destructor
         */
		virtual ~Image(){}

        /**
         * Load the image file
         *
         * @param filename The file name
		 * @return true if the file was loaded
         */
		bool load(const String &filename, uint32 &numberId);

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Image";}

	private:
        /**
         * The filename.
         */
		String _filename;

        /**
         * The image id.
         */
		uint32 _imageId;

		// image types

        /**
         * The OpenIL loader.
		 *
		 * @return true if the file was loaded
         */
		bool openDevil();
	};
}

#endif