#ifndef IO_BYTEARRAYINPUTSTREAM_H
#define IO_BYTEARRAYINPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
 
#include <io/InputStream.h>
#include <lang/Array.h>

namespace io
{
    /**
     * @brief
     * A ByteArrayInputStream contains an internal buffer that contains bytes that 
     * may be read from the stream. An internal counter keeps track of the next byte 
     * to be supplied by the read method.
     */
    class API_STATE ByteArrayInputStream : public InputStream
    {
    private:
        /**
         * The data buffer
         */
        lang::Array<char> _buffer;

        /**
         * The data index
         */
        int32 _index;

		/**
		 * Position
		 */
		int32 _position;

    public:

        /**
         * Constructor 
         */
        ByteArrayInputStream(const void* pData, int32 size);

        /**
         * Destructor
         */
        virtual ~ByteArrayInputStream();

        /**
         * Resets with new input buffer.
         * Note that the contents of the buffer is duplicated so the buffer
         * can be freed immediately after ByteArrayInputStream constructor returns.
         */
        void reset(const void* pData, int32 size);

        /**
         * @see InputStream::read()
         */
        int32 read();

        /**
         * @see InputStream::read(void* pDataOut, int32 size)
         */
        int32 read(char *pDataOut, int32 size);

        /**
         * @see InputStream::int32 skip(int32 n)
         */
        int32 skip(int32 n);

        /**
         * @see InputStream::available()
         */
        int32 available() const;

		/**
		 * @see Object::toString()
		 */
		const char *toString();
    };
} //namespace

#endif
