#ifndef IO_BYTEARRAYOUTPUTSTREAM_H
#define IO_BYTEARRAYOUTPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
 
#include <io/OutputStream.h>
#include <lang/Array.h>

namespace io
{
    /**
     * @brief
     * This class implements an output stream in which the data is written into a byte array. 
     * The buffer automatically grows as data is written to it. The data can be retrieved using 
     * toByteArray() and toString().
     */
    class API_STATE ByteArrayOutputStream : public OutputStream
    {
    private:
        /**
         * The buffer
         */
        lang::Array<char> _buffer;

    public:
        /**
         * Constructor
         */
        ByteArrayOutputStream(int32 size = 0);

        /**
         * Destructor
         */
        virtual ~ByteArrayOutputStream();

        /**
         * Resets the count field of this byte array output stream to 
         * zero, so that all currently accumulated output in the ouput stream is discarded. 
         */
        void reset();

        /**
         * Returns the current size of the buffer.
         */
        int32 size() const;

        /**
         * Creates a newly allocated byte array.
         */
        const char*	toByteArray() const;

        /**
         * Returns byte array identifier. 
         */
        lang::String toNatString() const;

        /**
         * Writes the complete contents of this byte array output stream to 
         * the specified output stream argument, as if by calling the output 
         * stream's write method using out.write(buf, count).
         * 
         * @param out The output outputstream
         */
        void writeTo(OutputStream *pStreamOut);

        /**
         * @see OutputStream::write(const char data)
         */
        void write(const char data);

        /**
         * @see OutputStream::write(const lang::String& data)
         */
        void write(const lang::String& data);

        /**
         * @see OutputStream::write(const void* data, int32 size)
         */
        void write(const char *pData, int32 size);

        /**
         * @see Object::toString()
         */
        const char *toString();
    };
} //namespace


#endif
