#ifndef IO_FILEINPUTSTREAM_H
#define IO_FILEINPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <io/InputStream.h>
#include <stdio.h>

namespace io
{
    /**
     * @brief
     * A FileInputStream obtains input bytes from a file in a file system. 
     * What files are available depends on the host environment.
     * 
     * @note
     * FileInputStream is meant for reading streams of raw bytes such as 
     * image data. For reading streams of characters, consider using FileReader.
     */
    class API_STATE FileInputStream : public InputStream
    {
    private:
        /**
         * The file data handle
         */
        FILE* _fileHandle;

    public:
        /**
         * Constructor 
         */
        FileInputStream();

        /**
         * Destructor
         */
        virtual ~FileInputStream();

        /**
         * Open the desired file.
         * 
         * @param filename The name of file
         * @return true if operation was a success.
         */
        bool openFile(const lang::String& filename);

        /**
         * @see InputStream::read()
         */
        int32 read();

        /**
         * @see InputStream::read()
         */
        int32 read(uint16 &data);

        /**
         * @see InputStream::read(void* pDataOut, int32 size)
         */
        int32 read(char *pDataOut, int32 size);

        /**
         * @see InputStream::int32 skip(int32 n)
         */
        int32 skip(int32 n);

        /**
         * @see InputStream::available()
         */
        int32 available() const;

		/**
		 * @see Object::toString()
		 */
		const char *toString();
    };
} // io


#endif
