#ifndef IO_FILEOUTPUTSTREAM_H
#define IO_FILEOUTPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <io/OutputStream.h>
#include <stdio.h>

namespace io
{
    /**
     * @brief
     * A file output stream is an output stream for writing data to a File 
     * or to a FileDescriptor. Whether or not a file is available or may be 
     * created depends upon the underlying platform. Some platforms, in particular, 
     * allow a file to be opened for writing by only one FileOutputStream 
     * (or other file-writing object) at a time. In such situations the constructors 
     * in this class will fail if the file involved is already open.
     * 
     * @note
     * FileOutputStream is meant for writing streams of raw bytes 
     * such as image data. We use write(string) to work with strings.
     */
    class API_STATE FileOutputStream : public OutputStream
    {
    private:
        /**
         * The file data handle
         */
        FILE* _fileHandle;

    public:
        /**
         * Constructor
         */
        FileOutputStream(const lang::String& filename);

        /**
         * Destructor
         */
        virtual ~FileOutputStream();

        /**
         * Returns name of the file 
         */
        lang::String toString() const;

        /**
         * Close the file.
         */
        void close();

        /**
         * @see OutputStream::write(const int32 data)
         */
        void write(const int32 data);

        /**
         * @see OutputStream::write(const char data)
         */
        void write(const char data);

        /**
         * @see OutputStream::write(const lang::String& data)
         */
        void write(const lang::String& data);

        /**
         * @see OutputStream::write(const void* data, int32 size)
         */
        void write(const char* pData, int32 size);

		/**
		 * @see Object::toString()
		 */
		const char *toString();
    };
} // io


#endif
