#ifndef IO_INPUTSTREAM_H
#define IO_INPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>
#include <util/Log.h>
#include <lang/String.h>

namespace io
{
    /**
     * @brief
     * This abstract class is the superclass of all classes representing an input stream of bytes.
     * 
     * @note
     * Applications that need to define a subclass of InputStream  must always provide a method 
     * that returns the next byte of input.
     */
	class API_STATE InputStream : public Object
    {
    public:
        /**
         * Returns a byte of data from this input stream.
         */
        virtual int32 read() = 0;

        /**
         * Tries to read specified number of bytes from the stream.
         * Doesn't block the caller if specified number of bytes isn't available.
         * 
         * \note
         * We don't use the read(byte[] b) 
         *
         * @param pDataOut The data buffer
         * @param size     The size of buffer
         * @return Number of bytes actually read.
         */
        virtual int32 read(char *pDataOut, int32 size) = 0;

        /**
         * Tries to skip over n bytes from the stream.
         * 
         * @param n The number to be skipped
         * @return Number of bytes actually skipped.
         */
        virtual int32 skip(int32 n) = 0;

        /**
         * Returns the number of bytes that can be read from the stream without blocking.
         */
        virtual int32 available() const = 0;
    };
} //namespace

#endif

