#ifndef IO_OUTPUTSTREAM_H
#define IO_OUTPUTSTREAM_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>
#include <util/Log.h>
#include <lang/String.h>

namespace io
{
    /**
     * @brief
     * This abstract class is the superclass of all classes representing an output stream of bytes. 
     * An output stream accepts output bytes and sends them to some sink.
     */
    class API_STATE OutputStream : public Object
    {
    public:
        /**
         * Flushes this output stream and forces any buffered output bytes to be written out.
         */
        virtual void flush()
        {}

        /**
         * Writes the specified byte to this byte array output stream.
         */
        virtual void write(const char data) = 0;

        /**
         * Writes a string in the array output stream.
         * 
         * \note
         * With this method we don't need to use the FileWriter for example.
         * 
         * @param data Data to be write
         */
        virtual void write(const lang::String& data) = 0;

        /**
         * Writes len bytes from the specified byte array to this output stream.
         * 
         * @param data Data to be write
         * @param size Size of bytes
         */
        virtual void write( const char* pData, int32 size ) = 0;
    };
} //namespace

#endif
