/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

template <class T>
Array<T>::Array() 
: _pBuffer(NULL), _arraySize(0), _length(0)
{ }

template <class T>
Array<T>::Array(const Array<T>& in) 
: _pBuffer(NULL), _arraySize(0), _length(0)
{
    reserve(in.getLength());

    for(uint16 i=0; i < in.getLength(); i++ )
    {
        add(in[i]);
    }
}

template <class T>
Array<T>::~Array()
{
    delete[] _pBuffer;
}

template <class T>
void Array<T>::resize()
{
    if (_length==_arraySize)
    {
        if (_arraySize==0)
        {
            _arraySize=4;
        }
        else
        {
            _arraySize+=(_arraySize*3)>>1;
        }

        T *tmp=new T[_arraySize];
        assert(tmp);
        //memset(tmp, 0, _arraySize);

        for(uint16 i=0; i < getLength(); ++i)
        {
            tmp[i] = _pBuffer[i];
        }

        delete[] _pBuffer;
        _pBuffer = tmp;
    }
}

template <class T>
uint32 Array<T>::getLength() const
{
    return _length;
}

template <class T>
uint32 Array<T>::getSize() const
{
    return _arraySize * _length;
}

template <class T>
void Array<T>::reserve(uint32 number)
{
    assert(number > 0);

    if (_arraySize < number)
    {
        T * tmp = new T[number];
        assert(tmp);
        _arraySize = number;
        //memset(tmp, 0, _arraySize);

        for(uint32 i=0; i < _length; ++i)
        {
            tmp[i] = _pBuffer[i];
        }

        delete[] _pBuffer;
        _pBuffer = tmp;
    }
}

template <class T>
void Array<T>::free()
{
    _pBuffer=0;
    _length=0;
    _arraySize=0;
}

template <class T>
void Array<T>::clear()
{
    _length=0;
}

template <class T>
void Array<T>::add(const T elem)
{
    resize();

    _pBuffer[_length++]=elem;
}

template <class T>
void Array<T>::remove(int32 i)
{
    if (i < _length)
    {
        for (int32 j = i; j < _length-1; j++)
        {
            _pBuffer[j] = _pBuffer[j+1];
        }
        _length--;
    }
}

template <class T>
void Array<T>::remove(int32 i, int32 n)
{
    if (i+n<=_length)
    {
        for (int32 j = i; j < _length-n; j++)
        {
            _pBuffer[j] = _pBuffer[j+n];
        }
        _length-=n;
    }
}

template <class T>
const T& Array<T>::operator[](int32 i) const
{
    return(_pBuffer[i]);
}

template <class T>
T& Array<T>::operator[](int32 i)
{
    return _pBuffer[i];
}

template <class T>
void Array<T>::operator=(const Array<T>& in)
{
    reserve(in.getLength());
    clear();

    for(uint16 i=0; i < in.getLength(); i++)
    {
        add(in[i]);
    }
}

template <class T>
void Array<T>::operator+=(Array<T>& in)
{	
    if (_length+in.getLength()>_arraySize)
    {
        _arraySize=getLength()+in.getLength();
        T *tmp=new T[_arraySize];
        assert(tmp);

        for(uint16 i=0; i < getLength(); i++)
        {
            tmp[i]=_pBuffer[i];
        }

        delete[] _pBuffer;
        _pBuffer=tmp;
    }

    for(uint16 i=0; i < in.getLength(); i++)
    {
        _pBuffer[i+getLength()]=in[i];
    }
    _length+=in.getLength();
}

template <class T>
const char * Array<T>::toString()
{
    return "class: Array";
}

