#ifndef MATH_MATRIX_H_
#define MATH_MATRIX_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
 
#include <math/Vector3.h>

namespace math
{
    /**
     * @brief
   	 * The Matrix Class.
   	 */
    class API_STATE Matrix : public Object
    {
    public:
        float32 m[4][4]; //!< matrix elements

        /**
         * Constructor
         */
        Matrix();

        /**
         * Copy-Constructor
         * 
         * @param in Matrix reference
         */
        Matrix(const Matrix& in);

        /**
         * Destructor
         */
        ~Matrix();

        /**
         * Reset all elements
         */
        void reset();

        /**
         * Load the identity matrix
         */
        void loadIdentity();

        /**
         * Set the matrix as the rotation matrix of angle given by 'ang' around direction 'dir'
         * 
         * @param ang Angle
         * @param dir Direction
         */
        void setRotation(float32 ang, const Vector3f& dir);

        /**
         * Multiply the matrix by another with rotation given by 'ang' around direction 'dir'
         * 
         * @param and Angle
         * @param dir Direction
         */
        void rotate( float32 ang, const Vector3f& dir );

        /**
         * Interpolate between two rotation matrices using quaternions
         * 
         * @param matrix1 The first matrix
         * @param matrix2 The second matrix
         * @param interValue Interpolation value
         */
        void lerp(const Matrix& matrix1, const Matrix& matrix2, float32 interValue);

        /**
         * Atribuition operator
         * 
         * @param in Matrix reference
         */
        void operator=(const Matrix& in);

        /**
         * Multiplication operator
         * 
         * @param in The matrix reference
         */
        Matrix operator*(const Matrix& in) const;

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: Matrix";}
    };
} //namespace
#endif
