#ifndef __MATHUTIL__
#define __MATHUTIL__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <math.h>
#include <stdlib.h>

namespace math
{
    /** lower bound for overflow detection*/
    static const float32 AK_BOUND_SMALL = 1.0e-4f;

	/** upper bound for overflow detection*/
    static const float32 AK_BOUND_BIG = 1.0e+10f;

    /** represents the value of pi*/
    static const float32 AK_PI = 3.14159265359f;

    /** represents the value of 2*pi*/
	static const float32 AK_PI_DOUBLE = 6.28318530718f;

	/** represents the value of pi/2*/
    static const float32 AK_PI_HALF = 1.57079632679f;

    /** represents the value of pi/180*/
    static const float32 AK_PI_OVER_180 = 1.74532925199433E-002f;

	/** represents the value of 180/pi*/
    static const float32 AK_PI_UNDER_180 = 5.72957795130823E+001f;

    /**
     * Random floating point number
     * 
     * @note
     * 
     * 
     * @return Returns the random number
     */
    inline float32 floatRandom()
    {
        return (((rand() % 10000) - 5000) / 5000.0f);
    }
}

#endif