#ifndef __VECTOR3__
#define __VECTOR3__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Log.h>
#include <util/Object.h>

namespace math
{
    /**
     * @brief
     * The vector3 class.
     */
	template <class T>
	class API_STATE Vector3 : public Object
	{
	public:
        /**
         * Predefined vector property
         */
		static const Vector3 UNIT_ZERO;
		static const Vector3 UNIT_ONE;
		static const Vector3 UNIT_X;
		static const Vector3 UNIT_Y;
		static const Vector3 UNIT_Z;

        /**
         * x value
         */
		T x;

		/**
         * y value
         */
		T y;

		/**
         * z value
         */
		T z;

        /**
         * Constructor
         */
		Vector3();

		/**
         * Constructor
         */
		Vector3(T x, T y, T z);

        /**
         * Vectors comparison
         * 
         * @param vector The outside vector
		 * @return result
         */
		bool operator== (const Vector3& vector) const;
		bool operator!= (const Vector3& vector) const;
		bool operator<  (const Vector3& vector) const;
		bool operator<= (const Vector3& vector) const;
		bool operator>  (const Vector3& vector) const;
		bool operator>= (const Vector3& vector) const;

        /**
         * Vectors operation
         * 
         * @param vector The outside vector
		 * @return the vector result
         */
		Vector3<T> operator+ (const Vector3& vector) const;
		Vector3<T> operator- (const Vector3& vector) const;
		Vector3<T> operator* (T value) const;
		Vector3<T> operator/ (T value) const;

        /**
         * Vectors attribution
         * 
         * @param vector The outside vector
		 * @return the vector result
         */
		Vector3<T>& operator+= (const Vector3& vector);
		Vector3<T>& operator-= (const Vector3& vector);
		Vector3<T>& operator*= (T value);
		Vector3<T>& operator/= (T value);

		/**
         * Negate the vector values
         * 
		 * @return the vector result
         */
		Vector3<T> negative() const;

		/**
         * Normalize the vector values
         */
		void normalize();

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: Vector3";}
	};

	typedef Vector3<float> Vector3f;
}

#endif