#ifndef __CAMERA_H__
#define __CAMERA_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Transformation.h>
#include <math/Vector3.h>


namespace scene
{
    /**
     * @brief
     * This is the Camera class.
     */
	class API_STATE Camera : public Transformation
	{
	public:
        /**
         * Constructor
         */
		Camera();

        /**
         * Destructor
         */
		virtual ~Camera() { }

        /**
         * Initialize the camera properties
		 *
		 * @note Use this method before use the camera
         */
		void initialize();

        /**
         * Set the up vector
         *
		 * @param upVector The up vector value
         */
		void setUpVector(const math::Vector3f &upVector);

        /**
         * Set the camera perspective
         *
		 * @param fovValue The fov value
		 * @param nearValue The near value
		 * @param farValue The far value
         */
		void setPerspective(float32 fovValue, float32 nearValue, float32 farValue);

        /**
         * Enable the camera
         *
		 * @param value Set to enable the camera
         */
		void setEnable(bool value);

		/**
         * Verify if the camera is enabled
         *
		 * @return True if enabled
         */
		bool isEnabled();

		/**
         * Update the camera data
         */
		void update();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Camera";}

	private:
		/**
         * Used to enable the camera
         */
		bool _enabled;

		/**
         * The fov value
         */
		float32 _fovValue;

		/**
         * The far value
         */
		float32 _farValue;

		/**
         * The near value
         */
		float32 _nearValue;

		/**
         * The upvector value
         */
		math::Vector3f _upVector;
	};

#include <scene/Camera.inl>
}

#endif