#ifndef __LIGHTNODE_H__
#define __LIGHTNODE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Node.h>

namespace scene
{
	class Light;

    /**
     * @brief
     * This is the light node class.
     */
	class API_STATE LightNode : public Node
	{
	public:
        /**
         * Constructor
         */
		LightNode();

        /**
         * Destructor
         */
		virtual ~LightNode();

        /**
         * @see Node::draw()
         */
		virtual void draw(render::Render *pRender);

        /**
         * Set the light on the node
         *
		 * @param pLight The light object
         */
		void setLight(Light *pLight);

        /**
         * Get the light on the node
         *
		 * @return the light
         */
		const Light* getLight() const;
		Light* getLight();

	private:
        /**
         * The light object
         */
		Light *_pLight;
	};

#include <scene/LightNode.inl>
}
#endif