#ifndef __MESHNODE_H__
#define __MESHNODE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Node.h>

namespace scene
{
	class Mesh;

    /**
     * @brief
     * This is the mesh node class.
     */
	class API_STATE MeshNode: public Node
	{
	public:
        /**
         * Constructor
         */
		MeshNode();

        /**
         * Destructor
         */
		virtual ~MeshNode();

        /**
         * @see Node::draw()
         */
		virtual void draw(render::Render *pRender);

        /**
         * Set the mesh on the node
         *
		 * @param pModel The mesh object
         */
		void setMesh(Mesh *pModel);

        /**
         * Get the mesh on the node
         *
		 * @return the mesh
         */
		const Mesh* getMesh() const;
		Mesh* getMesh();

	private:
        /**
         * The mesh object
         */
		Mesh *_pModel;

        /**
         * A rotation object vector (used to get a temporary rotation value )
         */
		math::Vector3f _rotate;

        /**
         * A rotation angle value (used to get a temporary rotation value )
         */
		float _rotateAngle;
	};

#include <scene/MeshNode.inl>
}

#endif