#ifndef __TRANSFORMATION_H__
#define __TRANSFORMATION_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>
#include <math/Vector3.h>


namespace scene
{
    /**
     * @brief
     * This is the Transformation class.
     */
	class API_STATE Transformation : public Object
	{
	public:
        /**
         * Constructor
         */
		Transformation();

        /**
         * Destructor
         */
		virtual ~Transformation() { }

        /**
         * Set the translation
         *
		 * @param translate The translation value
         */
		void setTranslation(const math::Vector3f &translate);

        /**
         * Set the target
         *
		 * @param target The target value
         */
		void setTarget(const math::Vector3f &target);

        /**
         * Set the rotation
         *
		 * @param rotateAngle The rotate angle
		 * @param rotate The rotate 
         */
		void setRotation(float rotateAngle, const math::Vector3f &rotate);

        /**
         * Set the scale
         *
		 * @param scale The scale
         */
		void setScale(math::Vector3f);

        /**
         * Get the translation
         *
		 * @return The translation
         */
		const math::Vector3f &getTranslation();

        /**
         * Get the rotation
         *
		 * @param rotateAngle Return the rotate angle
		 * @param rotate Return the rotate 
         */
		void getRotation(float &rotateAngle, math::Vector3f &rotate);

        /**
         * Get target
         *
		 * @return The scale
         */
		const math::Vector3f &getTarget();

		/**
         * Get scale
         *
		 * @return The scale
         */
		const math::Vector3f &getScale();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Transformation";}

	protected:
		/**
         * The translation value
         */
		math::Vector3f _translate;

		/**
         * The target value
         */
		math::Vector3f _target;

		/**
         * The rotation
         */
		math::Vector3f _rotate;

		/**
         * The rotate angle
         */
		float _rotateAngle;

		/**
         * The scale
         */
		math::Vector3f _scale;
	};

#include <scene/Transformation.inl>
}

#endif