#ifndef __ALPHAPROPERTY_H__
#define __ALPHAPROPERTY_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/property/Property.h>

namespace scene
{
    /**
     * @brief
     * This is the Alpha property class.
     */
	class API_STATE AlphaProperty : public Property
	{
	public:
		/**
		 * The Alpha properties.
		 */
		enum E_BLEND_FUNCTION
		{
			BF_ZERO = 0,
			BF_ONE,
			BF_DST_COLOR,
			BF_SRC_ALPHA,
			BF_DST_ALPHA,
			BF_ONE_MINUS_DST_COLOR,
			BF_ONE_MINUS_SRC_ALPHA,
			BF_ONE_MINUS_DST_ALPHA,
			BF_SRC_ALPHA_SATURATE,
			BF_SRC_COLOR,
			BF_ONE_MINUS_SRC_COLOR
		};

        /**
         * Constructor
         */
		AlphaProperty();

        /**
         * Destructor
         */
		virtual ~AlphaProperty(){}

        /**
         * Set the source alpha property
         *
		 * @param eValue The property value
         */
		void setSourceFunction(E_BLEND_FUNCTION eValue);

        /**
         * Set the destination alpha property
         *
		 * @param eValue The property value
         */
		void setDestinationFunction(E_BLEND_FUNCTION eValue);

        /**
         * Set the source and destination alpha property
         *
		 * @param eSrc The source property value
		 * @param eDest The destination property value
         */
		void setSrcAndDestFunction(E_BLEND_FUNCTION eSrc, E_BLEND_FUNCTION eDest);

        /**
         * Get the source alpha property
         *
		 * @return The property
         */
		E_BLEND_FUNCTION getSourceFunction();

        /**
         * Get the destination alpha property
         *
		 * @return The property
         */
		E_BLEND_FUNCTION getDestinationFunction();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: AlphaProperty";}

	private:
		/**
		 * The source value
		 */
		E_BLEND_FUNCTION _eSourceValue;

		/**
		 * The destination value
		 */
		E_BLEND_FUNCTION _eDestinationValue;
	};

#include <scene/property/AlphaProperty.inl>
}

#endif