#ifndef __MATERIALPROPERTY_H__
#define __MATERIALPROPERTY_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/property/Property.h>
#include <scene/ColorRGBA.h>

namespace scene
{
    /**
     * @brief
     * This is the Material property class.
     */
	class API_STATE MaterialProperty : public Property
	{
	public:
        /**
         * Constructor
         */
		MaterialProperty();

        /**
         * Destructor
         */
		virtual ~MaterialProperty(){}

        /**
         * Set the emissive material
         *
		 * @param emissive The color value
         */
		void setEmissive(ColorRGBA &emissive);

        /**
         * Set the ambient material
         *
		 * @param ambient The color value
         */
		void setAmbient(ColorRGBA &ambient);

        /**
         * Set the diffuse material
         *
		 * @param diffuse The color value
         */
		void setDiffuse(ColorRGBA &diffuse);

        /**
         * Set the specular material
         *
		 * @param specular The color value
         */
		void setSpecular(ColorRGBA &specular);

        /**
         * Set the shininess material
         *
		 * @param shininess The color value
         */
		void setShininess(float32 shininess);

        /**
         * Set the alpha value
         *
		 * @param value The alpha value
         */
		void setAlphaValue(float32 value);

        /**
         * Get the emissive material
         *
		 * @return The material
         */
		const ColorRGBA &getEmissive();

        /**
         * Get the ambient material
         *
		 * @return The material
         */
		const ColorRGBA &getAmbient();

        /**
         * Get the diffuse material
         *
		 * @return The material
         */
		const ColorRGBA &getDiffuse();

        /**
         * Get the specular material
         *
		 * @return The material
         */
		const ColorRGBA &getSpecular();

        /**
         * Get the shininess material
         *
		 * @return The material
         */
		float32 getShininess();

        /**
         * Get the alpha value
         *
		 * @return The material
         */
		float32 getAlphaValue();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: MaterialProperty";}

	private:
		/**
		 * The emissive value
		 */
		ColorRGBA _emissive;

		/**
		 * The ambient value
		 */
		ColorRGBA _ambient;

		/**
		 * The diffuse value
		 */
		ColorRGBA _diffuse;

		/**
		 * The specular value
		 */
		ColorRGBA _specular;

		/**
		 * The shininess value
		 */
		float32 _shininess;

		/**
		 * The alpha value
		 */
		float32 _alphaValue;
	};

#include <scene/property/MaterialProperty.inl>
}

#endif