#ifndef __PROPERTY_H__
#define __PROPERTY_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Object.h>
#include <util/Log.h>

namespace scene
{
    /**
     * @brief
     * This is the property class.
     */
	class API_STATE Property : public Object
	{
	public:
		/**
		 * The properties type.
		 */
		enum E_PROPERTY_TYPE
		{
			PT_UNKNOWN = 0,
			PT_ALPHA,
			PT_MATERIAL,
			PT_TEXTURE,
			PT_CULL
		};

        /**
         * Constructor
		 *
		 * @param type The property type
         */
		Property(E_PROPERTY_TYPE type = PT_UNKNOWN);

        /**
         * Destructor
         */
		virtual ~Property(){}
		
        /**
         * Get the property type
         *
		 * @return The property
         */
		E_PROPERTY_TYPE getType();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Property";}

	private:
		/**
		 * The property value
		 */
		E_PROPERTY_TYPE _type;
	};

#include <scene/property/Property.inl>

}

#endif