#ifndef __TEXTUREPROPERTY_H__
#define __TEXTUREPROPERTY_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/property/Property.h>

namespace scene
{
    /**
     * @brief
     * This is the Texture property class.
     */
	class API_STATE TextureProperty : public Property
	{
	public:
        /**
         * Constructor
         */
		TextureProperty();

        /**
         * Destructor
         */
		virtual ~TextureProperty(){}

        /**
         * Set the texture data
         *
		 * @param data The texture data
         */
		void setTextureData(const void * data);

        /**
         * Set the texture width
         *
		 * @param value The texture width
         */
		void setWidth(uint32 value);

        /**
         * Set the texture height
         *
		 * @param value The texture height
         */
		void setHeight(uint32 value);

        /**
         * Set the texture format
         *
		 * @param value The texture format
         */
		void setFormat(uint32 value);

        /**
         * Set the texture channels
         *
		 * @param value The texture channels
         */
		void setChannels(uint32 value);

        /**
         * Get the texture data
         *
		 * @return The texture data
         */
		const void * getTextureData();

        /**
         * Get the texture width
         *
		 * @return The texture witdh
         */
		uint32 getWidth();

        /**
         * Get the texture height
         *
		 * @return The texture height
         */
		uint32 getHeight();

        /**
         * Get the texture format
         *
		 * @return The texture format
         */
		uint32 getFormat();

        /**
         * Get the texture channels
         *
		 * @return The texture channels
         */
		uint32 getChannels();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: TextureProperty";}

	private:
		/**
		 * The texture data
		 */
		void * _textureData;

		/**
		 * The texture width
		 */
		uint32 _width;

		/**
		 * The texture height
		 */
		uint32 _height;

		/**
		 * The texture format
		 */
		uint32 _format;

		/**
		 * The texture channels
		 */
		uint32 _channels;
	};

#include <scene/property/TextureProperty.inl>

}

#endif