#ifndef __ASSERT_H__
#define __ASSERT_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
#include <util/Define.h>
#include <util/Types.h>

// Assert macro
#undef assert
#if (ASSERTION_LOG == ON)
#   define assert(value) Assert::assertState(value, __FILE__, __LINE__);
#else
#   define assert(value)
#endif

/**
 * @brief 
 * Description: Assertions.
 * 
 * @note
 *  See the Define.h to know how turn ON the assetion.
 * 
 * Usage:
 * @code
 *   01) Pointer assert:
 *       assert(pointer);
 *
 *   02) Bool assert:
 *       assert(int32 > 0);
 *
 *   03) Pointer and bool assert:
 *       assert(pointer != NULL && int32 > 0);
 *
 * @endcode
 */
class API_STATE Assert
{
private:
    enum E_ASSERT_TYPE
    {
        AT_POINTER = 0,
        AT_BOOL
    };

    /**
     * This method is used to output assertions.
     * 
     * @param errorType   type of error.
     * @param pFile       the caller file.
     * @param line        the caller line.
     */
    static void printError(E_ASSERT_TYPE errorType, const char* pFile, int32 line);

public:

    /**
     * This method is used to catch the pointer assertion.
     * 
     * @param pState      the object assert statement.
     * @param pFile       the caller file.
     * @param line        the caller line.
     */
    static bool assertState(const void *pState, const char* pFile, int32 line);

    /**
     * This method is used to catch the bool assertion.
     * 
     * @param state       the bool assert statement.
     * @param pFile       the caller file.
     * @param line        the caller line.
     */
    static bool assertState(bool state, const char* pFile, int32 line);
};


#endif //__ASSERT_H__*/
