/* Aika
*
* Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <aika.h>
#include <core/Device.h>
#include <stdio.h>
#include <lang/String.h>
#include <util/Assert.h>

#include <scene/Scene.h>
#include <scene/CameraNode.h>
#include <scene/Camera.h>

#include <util/Ptr.h>
#include <math/Vector3.h>

#include <scene/Mesh.h>
#include <scene/MeshNode.h>

using namespace aika;

float lastTime = -1.0f;
float elapsedTime = 0.0f;
float framerate = 0.0f;
int click = 0;

void printData(int &click, render::Render *pRender, core::Device *dev)
{
	if ( lastTime == -1.0f )
	{
		lastTime = dev->getDeviceInstance()->getTimeInSeconds();
	}
	float fCurrentTime = dev->getDeviceInstance()->getTimeInSeconds();
	float fDelta = fCurrentTime - lastTime;
	lastTime = fCurrentTime;
	elapsedTime += fDelta;

	framerate = click / elapsedTime;

	pRender->writeText(10, 14, "Aika");
	lang::String strFrameRate = lang::String::intToStr((int)framerate);
	strFrameRate.concat(" fps");
	pRender->writeText(10, 13, strFrameRate.getChars());
}

void testEngine()
{
	Device::setDataPath("../media/");

	DeviceConfig config;
    config.pTitleName = "Hello World Sample";
    config.width = 800;
    config.height = 600;
    config.deviceType = core::DT_WIN32;
    config.renderType = render::RT_OPENGL;

	Ptr<Device> dev = kNew(Device);
	assert(dev);
    dev->createDevice(config);

    Render *rend = dev->getRenderInstance();
    assert(rend);

	Ptr<Scene> scene = kNew(Scene);
	scene->openScene("scene.aks");

	CameraNode *cameraNode = (CameraNode*)scene->getNodeByName("Camera");
	Camera *camera = (Camera*)cameraNode->getCamera();
	if(camera)
		camera->initialize();

	MeshNode *skyNode = (MeshNode*)scene->getNodeByName("Skydome");
	MeshNode *duckNode = (MeshNode*)scene->getNodeByName("Duck");

	InputMap *input = InputMap::getInstance();

	float incr = 0;
	float incr1 = 0;
	bool quitApp = false;
	while (dev->run() && quitApp == false)
    {
		if(input->verifyIfWasPressed(InputMap::KT_ESC))
			quitApp = true;

		incr+=0.01f;
		incr1 -= 0.1f;

        rend->begin(0.2f, 0.2f, 0.2f, 1.0f);

		scene->update();
		scene->draw(rend);

		if(camera)camera->update();

		if(skyNode)
			skyNode->getMesh()->setRotation(incr, math::Vector3f.UNIT_Y);

		if(duckNode)
			duckNode->getMesh()->setRotation(incr1, math::Vector3f.UNIT_Y);
		
		printData(click, rend, dev);

		rend->end();
		click++;
    }

    printf("\nInstances on: %d", Object::instances());
}

int main()
{
    testEngine();
    printf("\nInstances on: %d\n", Object::instances());
}




