/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <core/Device.h>
#include <core/DeviceWin32.h>
#include <render/RenderOpenGL.h>
//#include <render/RenderDirectx.h>

namespace core
{
	render::Render *Device::_renderInstance = NULL;
	Device *Device::_deviceInstace = NULL;
	float32 Device::windowWidth = NULL;
	float32 Device::windowHeight = NULL;
	lang::String Device::_dataPath;
	

    Device::Device() 
        : _pDeviceWindow(NULL), 
		_pRenderDevice(NULL)
    {
    }

    Device::~Device()
    {
        kDelete(_pDeviceWindow);
		kDelete(_pRenderDevice);
    }

	void Device::setDataPath(lang::String path)
	{
		_dataPath = path;
	}

	lang::String &Device::getDataPath()
	{
		return _dataPath;
	}

    bool Device::createWindow(DeviceConfig &device)
    {
        TRACE("Device::createWindow");

        return false;
    }

    bool Device::createDevice(DeviceConfig &device)
    {
        TRACE("Device::createDevice");

        switch (device.deviceType)
        {
        case core::DT_WIN32:
#ifdef WIN32
            _pDeviceWindow = kNew(DeviceWin32);
#else       
            Log::log << "Windows device was not compiled\n";
#endif
            break;

        case core::DT_LINUX:
            Log::log << "Linux device was not compiled\n";
            break;

        default:
            Log::log << "Device doesn't exist\n";
            break;
        }

		_deviceInstace = _pDeviceWindow;

        return (_pDeviceWindow) ? _pDeviceWindow->createWindow(device) : false;
    }

    bool Device::createRender(DeviceConfig &device, HWND hWnd)
    {
        TRACE("Device::createRender");

        switch (device.renderType)
        {
        case render::RT_OPENGL:
#if AK_OPENGL
            _pRenderDevice = kNew(render::RenderOpenGL);
#else
            Log::log << "OpenGL was not compiled\n";
#endif
            break;

        case render::RT_DIRECTX:
#if AK_DIRECTX
//            _pRenderDevice = kNew(render::RenderDirectx);
//#else
            Log::log << "Directx was not compiled\n";
#endif
            break;

        default:
            Log::log << "Type not defined\n";
            break;
        }

		if(_pRenderDevice == NULL)
			return false;

		_pRenderDevice->initialize(hWnd);

		windowWidth = device.width;
		windowHeight = device.height;
		_renderInstance = _pRenderDevice;

		return true;
    }

    bool Device::run()
    {
        return (_pDeviceWindow) ? _pDeviceWindow->run() : false;
    }

	Device *Device::getDeviceInstance()
	{
		return _deviceInstace;
	}

	uint32 Device::getRealKey(uint32 input)
	{
		uint32 key = 0;
		for(int i=0; i < InputMap::KT_QUANTITY; i++)
		{
			if(keyMap[i] == input)
			{
				key = i;
				break;
			}
		}

		return key;
	}

    render::Render *Device::getRenderInstance()
    {
		return _renderInstance;
    }
} //namespace
