/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <string.h>
#include <io/ByteArrayInputStream.h>
#include <util/Assert.h>

namespace io
{
    ByteArrayInputStream::ByteArrayInputStream( const void* pData, int32 size ) 
        : _index(0), _position(0)
    {
        assert(pData != NULL && size > 0);

        reset( pData, size );
    }

    ByteArrayInputStream::~ByteArrayInputStream()
    {
        _buffer.free();
    }

    void ByteArrayInputStream::reset(const void* pData, int32 size)
    {
        assert(pData != NULL && size > 0);

        _buffer.reserve(size);

        if (size > 0)
            memcpy(&_buffer[0], pData, size );
        _index = 0;
    }

    int32 ByteArrayInputStream::read()
    {
        return _buffer[_position++];
    }

    int32 ByteArrayInputStream::read(char *pDataOut, int32 size)
    {
        assert(pDataOut != NULL && size > 0);

        if(size > 0)
        {
            pDataOut = new char[size];
            memset(pDataOut, 0, size);

            for (int32 i = 0 ; i < size; ++i )
                pDataOut[i] = _buffer[i];

            pDataOut[size] = '\0';
        }

        return 0;
    }

    int32 ByteArrayInputStream::available() const
    {
        return (_buffer.getSize() - _index);
    }

    int32 ByteArrayInputStream::skip(int32 n)
    {
        // not implemented
        return 0;
    }

    const char * ByteArrayInputStream::toString()
    {
        return "class: ByteArrayInputStream";
    }
} //namespace
