/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <lang/String.h>
#include <util/Assert.h>

namespace lang
{
    String::String() 
        : _size(STR_MINSIZE), _length(0), _pBuffer(NULL)
    {
        _pBuffer = new char[_size];
        assert(_pBuffer);
        memset(_pBuffer, 0, _size);
    }

    String::String(const char* pData) 
        : _size(STR_MINSIZE), _length(strlen(pData)), _pBuffer(NULL)
    {
        assert(pData);

        while ( _size < _length + 1 )
            _size <<= 1;

        _pBuffer = new char[_size];
        assert(_pBuffer);
        memset(_pBuffer, 0, _size);

        strcpy(_pBuffer, pData);
    }

    String::String(const String& data) 
        : _size(data._size), _length(data._length), _pBuffer(NULL)
    {
        assert(data.length() > 0);

        _pBuffer = new char[_size];
        assert(_pBuffer);
        memset(_pBuffer, 0, _size);

        strcpy(_pBuffer, (const char *)data);
    }

    String::String(const char* pData, int32 p, int32 n) 
        : _size(STR_MINSIZE), _length(n), _pBuffer(NULL)
    {
        assert(pData);

        while ( _size < _length + 1 )
            _size <<= 1;

        _pBuffer = new char[_size];
        assert(_pBuffer);
        memset(_pBuffer, 0, _size);

        memcpy(_pBuffer, &pData[p], _length);
        _pBuffer[_length] = '\0';
    }

    String::~String()
    {
        delete[] _pBuffer;
    }

    const char* String::getChars() const
    {
        return _pBuffer;
    }

    void String::concat(const char* pData)
    {
        assert(pData);

        _length += strlen(pData) + 1;

        if (_size < _length + 1)
        {
            while ( _size < _length + 1 )
                _size <<= 1;

            char* temp = new char[_size];
            assert(temp);

			for(uint16 i=0; i < _length; ++i)
			{
				temp[i] = _pBuffer[i];
			}

			strcat(temp, pData);
            delete[] _pBuffer;

			_pBuffer = temp;
        }
        else
            strcat(_pBuffer, pData);
    }

    void String::concat(const String& in)
    {
        assert(in.length() > 0);

        concat((const char *)in);
    }

    int32 String::findString(const char *pData) const
    {
        assert(pData);

        char *pos=strstr(_pBuffer,pData);
        if (pos==0)
        {
            return -1;
        }

        return (int32)(pos-_pBuffer);
    }

    int32 String::findChar(char c) const
    {
        char *pos=strchr(_pBuffer,c);
        if (pos==0)
        {
            return -1;
        }

        return (int32)(pos-_pBuffer);
    }

    void String::setChar(int32 i, char c)
    {
        if (i<_length)
        {
            _pBuffer[i]=c;
        }
        if (c==0)
        {
            if (_length>i)
            {
                _length=i;
            }
        }
    }

    void String::clearChars(int32 n)
    {
        if (n<_length)
        {
            _length-=n;
            strcpy(_pBuffer,&_pBuffer[n]);
        }
    }

    String String::substringFixedNumber(int32 n) const
    {
        return String(_pBuffer, 0, n);
    }

    String String::substring(int32 p, int32 n) const
    {
        return String(_pBuffer, p, n);
    }

    bool String::equals(String &string)
    {
        assert(string.length() > 0);

        return (strcmp(_pBuffer, string.getChars()) == 0) ? true : false;
    }

    void String::toLowerCase()
    {
        _strlwr(_pBuffer);
    }

    void String::toUpperCase()
    {
        _strupr(_pBuffer);
    }

    char String::charAt(int32 i) const
    {
        return _pBuffer[i];
    }

    int32 String::length() const
    {
        return _length;
    }

    String String::intToStr(int32 i)
    {
        char buf[50];
        memset(buf, 0, 50);
        sprintf(buf, "%d", i);

        return String(buf);
    }

    //    ------------------------------------------------------

    void String::operator=(const String& in)
    {
        assert(in.length() > 0);

        operator=((const char *)in);
    }

    String String::operator+(const String& in)
    {
        assert(in.length() > 0);

        return operator+((const char *)in);
    }

    String String::operator+(const char* pData)
    {
        assert(pData);

        String str(_pBuffer);
        str.concat(pData);

        return str;
    }

    void String::operator=(const char* pData)
    {
        assert(pData);

        _length = strlen(pData);
        if (_size < _length + 1)
        {
            while ( _size < _length + 1 )
                _size <<= 1;

            delete[] _pBuffer;
            _pBuffer = new char[_size];
            assert(_pBuffer);
        }
        strcpy(_pBuffer, pData);
    }

    String::operator const char *() const
    {
        return _pBuffer;
    }

    int32 String::operator==(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)==0;
    }

    int32 String::operator!=(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)!=0;
    }

    int32 String::operator>(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)>0;
    }

    int32 String::operator<(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)<0;
    }

    int32 String::operator>=(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)>=0;
    }

    int32 String::operator<=(const char *pData) const
    {
        assert(pData);

        return strcmp(_pBuffer,pData)<=0;
    }

	bool String::compareStrings(const char* pStringOne, const char* pStringTwo)
	{
		assert(pStringOne && pStringTwo);

		if(strcmp(pStringOne, pStringTwo) == 0)
			return true;

		return false;
	}

    const char * String::toString()
    {
        return "class: String";
    }
} //namespace

