/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <math/Vector2.h>

namespace math
{
	template class API_STATE Vector2<float>;
	const Vector2f Vector2f::UNIT_ZERO(0.0f,0.0f);
	const Vector2f Vector2f::UNIT_ONE(1.0f,1.0f);
	const Vector2f Vector2f::UNIT_X(1.0f,0.0f);
	const Vector2f Vector2f::UNIT_Y(0.0f,1.0f);

	template <class T>
	Vector2<T>::Vector2() : x(0), y(0)
	{
	}

	template <class T>
	Vector2<T>::Vector2(T x, T y) :
	x(x), y(y)
	{
	}

	template <class T>
	bool Vector2<T>::operator== (const Vector2& vector) const
	{
		if(x == vector.x && y == vector.y)
			return true;

		return false;
	}

	template <class T>
	bool Vector2<T>::operator!= (const Vector2& vector) const
	{
		if(x == vector.x && y == vector.y)
			return false;

		return true;
	}

	template <class T>
	bool Vector2<T>::operator<  (const Vector2& vector) const
	{
		if(x < vector.x || y < vector.y)
			return true;

		return false;
	}

	template <class T>
	bool Vector2<T>::operator<= (const Vector2& vector) const
	{
		if(x <= vector.x || y <= vector.y)
			return true;

		return false;
	}

	template <class T>
	bool Vector2<T>::operator>  (const Vector2& vector) const
	{
		if(x > vector.x || y > vector.y)
			return true;

		return false;
	}

	template <class T>
	bool Vector2<T>::operator>= (const Vector2& vector) const
	{
		if(x >= vector.x || y >= vector.y)
			return true;

		return false;
	}

	template <class T>
	Vector2<T> Vector2<T>::operator+ (const Vector2& vector) const
	{
		Vector2<T> tempVec;
		tempVec.x = x + vector.x;
		tempVec.y = y + vector.y;
		return tempVec;
	}

	template <class T>
	Vector2<T> Vector2<T>::operator- (const Vector2& vector) const
	{
		Vector2<T> tempVec;
		tempVec.x = x - vector.x;
		tempVec.y = y - vector.y;
		return tempVec;
	}

	template <class T>
	Vector2<T> Vector2<T>::operator* (T value) const
	{
		Vector2<T> tempVec;
		tempVec.x = value * x;
		tempVec.y = value * y;
		return tempVec;
	}

	template <class T>
	Vector2<T> Vector2<T>::operator/ (T value) const
	{
		Vector2<T> tempVec;
		tempVec.x = value / x;
		tempVec.y = value / y;
		return tempVec;
	}

	template <class T>
	Vector2<T> Vector2<T>::negative() const
	{
		Vector2<T> tempVec;
		tempVec.x = -x;
		tempVec.y = -y;
		return tempVec;
	}

	template <class T>
	Vector2<T>& Vector2<T>::operator+= (const Vector2& vector)
	{
		x += vector.x;
		y += vector.y;
		return *this;
	}

	template <class T>
	Vector2<T>& Vector2<T>::operator-= (const Vector2& vector)
	{
		x -= vector.x;
		y -= vector.y;
		return *this;
	}

	template <class T>
	Vector2<T>& Vector2<T>::operator*= (T value)
	{
		x *= value;
		y *= value;
		return *this;
	}

	template <class T>
	Vector2<T>& Vector2<T>::operator/= (T value)
	{
		x /= value;
		y /= value;
		return *this;
	}
}