/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <math.h>
#include <math/Vector3.h>

namespace math
{
	template class API_STATE Vector3<float>;
	const Vector3f Vector3f::UNIT_ZERO(0.0f,0.0f,0.0f);
	const Vector3f Vector3f::UNIT_ONE(1.0f,1.0f,1.0f);
	const Vector3f Vector3f::UNIT_X(1.0f,0.0f,0.0f);
	const Vector3f Vector3f::UNIT_Y(0.0f,1.0f,0.0f);
	const Vector3f Vector3f::UNIT_Z(0.0f,0.0f,1.0f);

	template <class T>
	Vector3<T>::Vector3() : x(0), y(0), z(0)
	{
	}

	template <class T>
	Vector3<T>::Vector3(T x, T y, T z) :
	x(x), y(y), z(z)
	{
	}

	template <class T>
	bool Vector3<T>::operator== (const Vector3& vector) const
	{
		if(x == vector.x && y == vector.y && z == vector.z)
			return true;

		return false;
	}

	template <class T>
	bool Vector3<T>::operator!= (const Vector3& vector) const
	{
		if(x == vector.x && y == vector.y && z == vector.z)
			return false;

		return true;
	}

	template <class T>
	bool Vector3<T>::operator<  (const Vector3& vector) const
	{
		if(x < vector.x || y < vector.y || z < vector.z)
			return true;

		return false;
	}

	template <class T>
	bool Vector3<T>::operator<= (const Vector3& vector) const
	{
		if(x <= vector.x || y <= vector.y || z <= vector.z)
			return true;

		return false;
	}

	template <class T>
	bool Vector3<T>::operator>  (const Vector3& vector) const
	{
		if(x > vector.x || y > vector.y || z > vector.z)
			return true;

		return false;
	}

	template <class T>
	bool Vector3<T>::operator>= (const Vector3& vector) const
	{
		if(x >= vector.x || y >= vector.y || z >= vector.z)
			return true;

		return false;
	}

	template <class T>
	Vector3<T> Vector3<T>::operator+ (const Vector3& vector) const
	{
		Vector3<T> tempVec;
		tempVec.x = x + vector.x;
		tempVec.y = y + vector.y;
		tempVec.z = z + vector.z;
		return tempVec;
	}

	template <class T>
	Vector3<T> Vector3<T>::operator- (const Vector3& vector) const
	{
		Vector3<T> tempVec;
		tempVec.x = x - vector.x;
		tempVec.y = y - vector.y;
		tempVec.z = z - vector.z;
		return tempVec;
	}

	template <class T>
	Vector3<T> Vector3<T>::operator* (T value) const
	{
		Vector3<T> tempVec;
		tempVec.x = value * x;
		tempVec.y = value * y;
		tempVec.z = value * z;
		return tempVec;
	}

	template <class T>
	Vector3<T> Vector3<T>::operator/ (T value) const
	{
		Vector3<T> tempVec;
		tempVec.x = value / x;
		tempVec.y = value / y;
		tempVec.z = value / z;
		return tempVec;
	}

	template <class T>
	Vector3<T> Vector3<T>::negative() const
	{
		Vector3<T> tempVec;
		tempVec.x = -x;
		tempVec.y = -y;
		tempVec.z = -z;
		return tempVec;
	}

	template <class T>
	Vector3<T>& Vector3<T>::operator+= (const Vector3& vector)
	{
		x += vector.x;
		y += vector.y;
		z += vector.z;
		return *this;
	}

	template <class T>
	Vector3<T>& Vector3<T>::operator-= (const Vector3& vector)
	{
		x -= vector.x;
		y -= vector.y;
		z -= vector.z;
		return *this;
	}

	template <class T>
	Vector3<T>& Vector3<T>::operator*= (T value)
	{
		x *= value;
		y *= value;
		z *= value;
		return *this;
	}

	template <class T>
	Vector3<T>& Vector3<T>::operator/= (T value)
	{
		x /= value;
		y /= value;
		z /= value;
		return *this;
	}

	template <class T>
	void Vector3<T>::normalize()
    {
        float32 f = (float32)sqrt(x*x + y*y + z*z);

        if (f != 0.0f)
        {
            x /= f;
            y /= f;
            z /= f;
        }
    }
}
