/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <math/Vector4.h>

namespace math
{
	template class API_STATE Vector4<float>;
	const Vector4f Vector4f::UNIT_ZERO(0.0f,0.0f,0.0f,0.0f);
	const Vector4f Vector4f::UNIT_ONE(1.0f,1.0f,1.0f,1.0f);
	const Vector4f Vector4f::UNIT_X(1.0f,0.0f,0.0f,0.0f);
	const Vector4f Vector4f::UNIT_Y(0.0f,1.0f,0.0f,0.0f);
	const Vector4f Vector4f::UNIT_Z(0.0f,0.0f,1.0f,0.0f);
	const Vector4f Vector4f::UNIT_W(0.0f,0.0f,0.0f,1.0f);

	template <class T>
	Vector4<T>::Vector4() : x(0), y(0), z(0), w(0)
	{
	}

	template <class T>
	Vector4<T>::Vector4(T x, T y, T z, T w) :
	x(x), y(y), z(z), w(w)
	{
	}

	template <class T>
	bool Vector4<T>::operator== (const Vector4& vector) const
	{
		if(x == vector.x && y == vector.y && z == vector.z && w == vector.w)
			return true;

		return false;
	}

	template <class T>
	bool Vector4<T>::operator!= (const Vector4& vector) const
	{
		if(x == vector.x && y == vector.y && z == vector.z && w == vector.w)
			return false;

		return true;
	}

	template <class T>
	bool Vector4<T>::operator<  (const Vector4& vector) const
	{
		if(x < vector.x || y < vector.y || z < vector.z)
			return true;

		return false;
	}

	template <class T>
	bool Vector4<T>::operator<= (const Vector4& vector) const
	{
		if(x <= vector.x || y <= vector.y || z <= vector.z || w <= vector.w)
			return true;

		return false;
	}

	template <class T>
	bool Vector4<T>::operator>  (const Vector4& vector) const
	{
		if(x > vector.x || y > vector.y || z > vector.z || w > vector.w)
			return true;

		return false;
	}

	template <class T>
	bool Vector4<T>::operator>= (const Vector4& vector) const
	{
		if(x >= vector.x || y >= vector.y || z >= vector.z || w >= vector.w)
			return true;

		return false;
	}

	template <class T>
	Vector4<T> Vector4<T>::operator+ (const Vector4& vector) const
	{
		Vector4<T> tempVec;
		tempVec.x = x + vector.x;
		tempVec.y = y + vector.y;
		tempVec.z = z + vector.z;
		tempVec.w = w + vector.w;
		return tempVec;
	}

	template <class T>
	Vector4<T> Vector4<T>::operator- (const Vector4& vector) const
	{
		Vector4<T> tempVec;
		tempVec.x = x - vector.x;
		tempVec.y = y - vector.y;
		tempVec.z = z - vector.z;
		tempVec.w = w - vector.w;
		return tempVec;
	}

	template <class T>
	Vector4<T> Vector4<T>::operator* (T value) const
	{
		Vector4<T> tempVec;
		tempVec.x = value * x;
		tempVec.y = value * y;
		tempVec.z = value * z;
		tempVec.w = value * w;
		return tempVec;
	}

	template <class T>
	Vector4<T> Vector4<T>::operator/ (T value) const
	{
		Vector4<T> tempVec;
		tempVec.x = value / x;
		tempVec.y = value / y;
		tempVec.z = value / z;
		tempVec.w = value / w;
		return tempVec;
	}

	template <class T>
	Vector4<T> Vector4<T>::negative() const
	{
		Vector4<T> tempVec;
		tempVec.x = -x;
		tempVec.y = -y;
		tempVec.z = -z;
		tempVec.w = -w;
		return tempVec;
	}

	template <class T>
	Vector4<T>& Vector4<T>::operator+= (const Vector4& vector)
	{
		x += vector.x;
		y += vector.y;
		z += vector.z;
		w += vector.w;
		return *this;
	}

	template <class T>
	Vector4<T>& Vector4<T>::operator-= (const Vector4& vector)
	{
		x -= vector.x;
		y -= vector.y;
		z -= vector.z;
		w -= vector.w;
		return *this;
	}

	template <class T>
	Vector4<T>& Vector4<T>::operator*= (T value)
	{
		x *= value;
		y *= value;
		z *= value;
		w *= value;
		return *this;
	}

	template <class T>
	Vector4<T>& Vector4<T>::operator/= (T value)
	{
		x /= value;
		y /= value;
		z /= value;
		w /= value;
		return *this;
	}

}
