/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Camera.h>
#include <core/Device.h>

namespace scene
{
	Camera::Camera() : _enabled(false)
	{
		TRACE("Camera::Camera");

		_translate = math::Vector3f(0.0f, 0.0f, 2.0f);
		_target = math::Vector3f(0.0f, 0.0f, 0.0f);
		_upVector = math::Vector3f(0.0f, 1.0f, 0.0f);
	}

	void Camera::update()
	{
		if(_enabled)
		{
			render::Render *pRender = core::Device::getRenderInstance();
			pRender->setProjectionType(render::PT_MODELVIEW);
			pRender->loadIdentity();

			pRender->setRotation(_rotateAngle, _rotate);
			pRender->setCameraTranslation(_translate, _target, _upVector);
		}
	}

	void Camera::initialize()
	{
		TRACE("Camera::initialize");

		render::Render *pRender = core::Device::getRenderInstance();

		pRender->setProjectionType(render::PT_PROJECTION);
		pRender->loadIdentity();

		pRender->setCameraPerspective(_fovValue, _nearValue, _farValue);

		pRender->setProjectionType(render::PT_MODELVIEW);
		pRender->loadIdentity();
	}

	void Camera::setPerspective(float32 fovValue, float32 nearValue, float32 farValue)
	{
		TRACE("Camera::setPerspective");

		_fovValue = fovValue;
		_nearValue = nearValue;
		_farValue = farValue;
	}
}