/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Mesh.h>
#include <util/Assert.h>
#include <core/Device.h>

namespace scene
{
	Mesh::Mesh() : _imageId(0), _hasTexture(false), _hasMaterial(false)
    {
		TRACE("Mesh::Mesh");
    }

    Mesh::~Mesh()
    {
    }

    void Mesh::load(const String &filename)
    {
		TRACE("Mesh::load");

        assert(filename.length() > 0);

		//filename.clearChars(core::Device::getDataPath().length());
  //      String strTemp = filename.substring(filename.findChar('.')+1, filename.length());
		
        String strTemp = filename;
		strTemp.clearChars(core::Device::getDataPath().length());
		strTemp = strTemp.substring(strTemp.findChar('.')+1, strTemp.length());

		_filename = filename;
        if(strTemp.equals(String("akg")))
        {
			openAkg();
        }
    }

	void Mesh::draw(render::Render *pRender)
	{
		if(_hasTexture)
			pRender->bindImage(_imageId);

		pRender->setFeatureStatus(render::FT_CULL_FACE, true);
		pRender->setFeatureStatus(render::FT_DEPTH_TEST, true);

		if(_hasMaterial)
		{
			pRender->setMaterialProperty(&_materialProperty);

			if(_materialProperty.getAlphaValue() < 1.0f)
			{
				pRender->setFeatureStatus(render::FT_BLEND, true);
				pRender->setAlphaProperty(&_alphaProperty);

				_cullProperty.setCullType(CullProperty::CT_BACK);
				pRender->drawList(_listId, &_cullProperty);

				pRender->setFeatureStatus(render::FT_BLEND, false);
			}
			else
			{
				_cullProperty.setCullType(CullProperty::CT_BACK);
				pRender->drawList(_listId, &_cullProperty);
			}
		}
		else
			pRender->drawList(_listId);

	}
}