/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <Scene/Node.h>
#include <stdio.h>

namespace scene
{

	Node::Node() : _used(0), _growBy(1)
	{
		TRACE("Node::Node");
	}

	Node::~Node() 
	{ 
		for(uint32 i=0; i < _child.getLength(); i++)
		{
			kDelete(_child[i]);
		}
	}

	int32 Node::attachChild(Node *pChild)
	{
		TRACE("Node::attachChild");
		assert(pChild);

		pChild->setParent(this);
		_used++;

		int32 quantity = _child.getLength();
		for(int32 i=0; i < quantity; i++)
		{
			if(_child[i] == 0)
			{
				_child[i] = pChild;
				return i;
			}
		}

		_child.add(pChild);

		return quantity;
	}

	int32 Node::detachChild(Node *pChild)
	{
		TRACE("Node::detachChild");
		assert(pChild);

		if(pChild)
		{
			for(uint32 i=0; i < _child.getLength(); i++)
			{
				if(_child[i] == pChild)
				{
					pChild->setParent(NULL);
					_child[i] = NULL;
					_used--;
					return i;
				}
			}
		}

		return -1;
	}

	void Node::update()
	{
		for(uint32 i=0; i < _child.getLength(); i++)
		{
			_child[i]->update();
		}
	}

	const Node * Node::getNodeAt(uint32 value)
	{
		Node * node = NULL;

		if(_child.getLength() > 0)
			node = _child[value];

		return node;
	}

	const Node * Node::getNodeByName(const char *pName)
	{
		Node * node = NULL;

		for(uint32 i=0; i < _child.getLength(); i++)
		{
			if(String::compareStrings(_child[i]->getName().getChars(), pName))
				node = _child[i];
		}

		return node;
	}


	void Node::draw(render::Render *pRender)
	{
		assert(pRender);

		for(uint32 i=0; i < _child.getLength(); i++)
		{
			_child[i]->draw(pRender);
		}
	}
}