/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Log.h>
#include <io/FileOutputStream.h>

//-----------------------
// Static Initialization
//-----------------------

/**
* The output file stream. Only used when the MEMORY_FILE_LOG is ON.
*/
#if (MEMORY_LOG == ON)
io::FileOutputStream memoryFile("memory.txt");
#endif

/**
* The output file stream. Only used when the TRACE_LOG is ON.
*/
#if (TRACE_LOG == ON)
io::FileOutputStream debugFile("trace.txt");
#endif

/**
* The output file stream. Only used when the ASSERTION_LOG is ON.
*/
#if (TRACE_LOG == ON)
io::FileOutputStream assertFile("assertion.txt");
#endif

Log Log::log;

int32 Log::_count = 0;

int32 Log::_depth = 0;

//------------------------


Log::Log()
{
}

Log::Log(const char *pMethodName)
{
#if (TRACE_LOG == ON)

    _pMethodName = (char*)pMethodName;

    record(_depth*2, "-->", _pMethodName);
    ++_depth;

#endif
}

Log::~Log()
{
#if (TRACE_LOG == ON)

    --_depth;
    if(_depth >= 0)
        record(_depth*2, "<--", "");
#endif

    if(Object::instances() == 0)
        closeLogs();
}

Log& Log::operator << (char *pText)
{
    printf("%s", pText);
    return *this;
}

Log& Log::operator << (int32 value)
{
    printf("%d", value);
    return *this;
}

Log& Log::operator << (uint32 value)
{
    printf("%u", value);
    return *this;
}

Log& Log::operator << (bool value)
{
    printf("%s", value ? "true" : "false");
    return *this;
}

Log& Log::operator << (void *pText)
{
    printf("0x%08X", (uint32)pText);
    return *this;
}

void Log::record(int32 align, const char *pText, const char *pMethodName)
{
    /*int32 depth, int32 align,*/ 
#if (TRACE_LOG == ON)

    if(align > 0)
    {
        for(int32 i = 1; i <= align; i++)
        {
            debugFile.write(' ');
            //printf(" ");
        }
    }

    //printf("(%d) [trace]:%s <%s>\n", depth, action, methodName);
    debugFile.write("(");
    debugFile.write(_depth);
    debugFile.write(")");
    debugFile.write(pText);
    debugFile.write(" <");
    debugFile.write(pMethodName);
    debugFile.write("> \n");

#endif
}

void Log::memAlloc(char *pComponent, char *pFile, int32 line)
{
#if (MEMORY_LOG == ON)

    _count++;
    memoryFile.write(_count);
    memoryFile.write(" ["); memoryFile.write(pComponent);
    memoryFile.write("] - file: "); memoryFile.write(pFile);
    memoryFile.write(" - line: "); memoryFile.write(line);
    memoryFile.write('\n');

#endif
}

void Log::assertMsg(const char* pErrorType, const char* pFile, int32 line)
{
#if (ASSERTION_LOG == ON)

    assertFile.write(pErrorType);
    assertFile.write(" - Error in line: ");
    assertFile.write(line);
    assertFile.write(" in file: ");
    assertFile.write(pFile);
#endif
}

void Log::closeLogs()
{
#if (MEMORY_LOG == ON)
	memoryFile.write("Number of memory allocated: "); memoryFile.write(instances());
    memoryFile.write('\n');
#endif

#if (TRACE_LOG == ON)
    debugFile.close();
#endif
}

const char * Log::toString()
{
    return "class: Log";
}