/* Aika
*
* Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

#include <util/Object.h>
#include <util/Assert.h>
#include <util/Log.h>

#define NEW_STR "NEW"
#define DEL_STR "DEL"

int32 Object::_instanceCount = 0;

Object::Object()
{
}

Object::~Object()
{
}

void* Object::operator new(size_t size)
{
    void *pPointer = NULL;

    if(size != sizeof(Object))
    {
        pPointer = ::operator new(size);

        if (pPointer != NULL)
        {
            memset(pPointer, 0, size);  

            //Log::memAlloc(NEW_STR, __FILE__, __LINE__, (pPointer), (size));
        }
    }

    // extra check
    assert(pPointer);

    if(size == 0)
        size = 1;

    _instanceCount++;
    return pPointer;
}

void Object::operator delete(void *pPointer)
{
    if (pPointer != NULL)
    {
        ::operator delete(pPointer);
        pPointer = NULL;

        _instanceCount--;

        //Log::memAlloc(DEL_STR, __FILE__, __LINE__);
    }
}

bool Object::equals(Object *pObject)
{
    return (this == pObject);
}

int32 Object::instances()
{
    return _instanceCount;
}

void Object::showPointer()
{
    Log::log << "Pointer: " << ((void *)this) << "\n";
}

Object* Object::clone()
{
    // temporary copy reference
    return this;
}
