/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include "AppFramework.h"

AppFramework::AppFramework(const char* pTitle, int16 windowWidth, int16 windowHeight, bool fullscreen)
: _mainDevice(NULL), 
_mainRender(NULL), 
_lastTime(-1.0f), 
_timeInSeconds(0.0f),
_framerate(0.0f),
_click(0),
_quitApp(false)
{
    DeviceConfig config;
    config.pTitleName = pTitle;
    config.width = windowWidth;
    config.height = windowHeight;
#if defined(WIN32)
    config.deviceType = core::DT_WIN32;
#endif
    config.renderType = render::RT_OPENGL;
	config.fullScreen = fullscreen;

	_mainDevice = kNew(Device);
	assert(_mainDevice);
    _mainDevice->createDevice(config);

    _mainRender = _mainDevice->getRenderInstance();
    assert(_mainRender);
}

AppFramework::~AppFramework()
{
	kDelete(_mainDevice);
}

void AppFramework::calculateTime()
{
	if (_lastTime == -1.0f)
	{
		_lastTime = _mainDevice->getDeviceInstance()->getTimeInSeconds();
	}

	float32 currentTime = _mainDevice->getDeviceInstance()->getTimeInSeconds();
	float32 delta = currentTime - _lastTime;
	_lastTime = currentTime;

	_timeInSeconds += delta;

	_framerate = _click / _timeInSeconds;
}

void AppFramework::onLoop()
{
	onCreate();

	while (_mainDevice->run() && !_quitApp)
    {
		// calculate time, framerate, etc
		calculateTime();

		// begin the render properties
        _mainRender->begin(0.2f, 0.2f, 0.2f, 1.0f);

		// process the input buttons
		processInput();

		// process the update data
		onUpdate();

		// process the draw data
		onDraw();

		// end render
		_mainRender->end();

		// add click
		_click++;
    }
}

void AppFramework::quitApp()
{
	_quitApp = true;
}

float32 AppFramework::getFrameRate()
{
	return _framerate;
}

float32 AppFramework::getElapsedTime()
{
	return _timeInSeconds;
}

int main()
{
	AppFramework *appFramework = AppFramework::createApp();

	appFramework->onLoop();

	kDelete(appFramework);
}
