#ifndef __APPFRAMEWORK_H__
#define __APPFRAMEWORK_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <aika.h>
#include <core/Device.h>
#include <util/Ptr.h>

using namespace aika;

/**
 * @brief
 * This is app framework.
 */
class AppFramework
{
public:
    /**
     * Destructor.
     */
	virtual ~AppFramework();

    /**
     * Constructor of the app instance.
     */
	static AppFramework * createApp();

    /**
     * Process the input.
     */
	virtual void processInput() = 0;

    /**
     * Used to create data before the main loop.
     */
	virtual void onCreate() = 0;

    /**
     * Update data.
     */
	virtual void onUpdate() = 0;

    /**
     * Draw data.
     */
	virtual void onDraw() = 0;

    /**
     * The main loop.
     */
	virtual void onLoop();

protected:
    /**
     * The main device instance.
     */
	Device *_mainDevice;

    /**
     * The main render instance.
     */
	Render *_mainRender;

    /**
     * Constructor.
	 *
	 * @param pTitle The window title
	 * @param windowWidth The window width
	 * @param windowHeight The window height
	 * @param fullscreen Set the fullscreen mode
     */
	AppFramework(const char* pTitle, int16 windowWidth, int16 windowHeight, bool fullscreen);

    /**
     * Get the framerate.
	 *
	 * @return The framerate
     */
	float32 getFrameRate();

    /**
     * Get the elapsed time.
	 *
	 * @return The elapsed time
     */
	float32 getElapsedTime();

    /**
     * Quit the app.
     */
	void quitApp();

private:
    /**
     * Flag used to quit the main loop.
     */
	bool _quitApp;

    /**
     * The last time.
     */
	float32 _lastTime;

    /**
     * The elapsed time.
     */
	float32 _timeInSeconds;

    /**
     * The framerate.
     */
	float32 _framerate;

    /**
     * The click.
     */
	uint32 _click;

    /**
     * Used to calculate the elapsed time and framerate.
     */
	void calculateTime();
};

#endif