#ifndef __AKGFILE_H__
#define __AKGFILE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <vector>

/**
 * @brief
 * This is simplevector. Used to make the lib more generic and independent.
 */
class SimpleVector
{
public:
       SimpleVector() : x(0), y(0), z(0), w(0){}
       SimpleVector(float x, float y, float z, float w) : x(x), y(y), z(z), w(w) {}
       float x, y, z, w;
};

/**
 * @brief
 * This is Normal data
 */
class NormalData
{
public:
	unsigned int normalSize;
	std::vector<float> normalData;
	std::vector<unsigned int> normalIndices;
};

/**
 * @brief
 * This is Vertex data
 */
class VertexData
{
public:
	unsigned int vertexSize;
	std::vector<float> vertexData;
	std::vector<unsigned int> vertexIndices;
};

/**
 * @brief
 * This is Texture data
 */
class TextureData
{
public:
	std::string textureFileName;
	unsigned int imageQuantity;

	unsigned int textureSize;
	std::vector<float> textureData;
	std::vector<unsigned int> textureIndices;
};

/**
 * @brief
 * This is Effect data
 */
class EffectData
{
public:
       SimpleVector ambient;
       SimpleVector specular;
       SimpleVector diffuse;
       SimpleVector emission;
       float shininess;
       float transparence;
};

/**
 * @brief
 * This is Material data
 */
class MaterialData
{
public:
       unsigned int materialQuantity;
       std::vector<EffectData> materialData;
};

/**
 * @brief
 * This is Geometry data
 */
class GeometryData
{
public:
	VertexData vertex;
	NormalData normal;
	TextureData texture;
	MaterialData material;

	unsigned int indicesSize;
};

/**
 * @brief
 * This is the IO akg.
 */
class ioAkg
{
public:
    /**
     * Write the file
     *
	 * @param pFileName the filename
	 * @param geometry the geometry data
	 *
	 * @return true if write the file
     */
	bool writeFile(const char* pFileName, const GeometryData &geometry);

    /**
     * Open the file
     *
	 * @param pFileName the filename
	 * @param geometry Return the geometry data
	 *
	 * @return true if opened the file
     */
	bool openFile(const char* pFileName, GeometryData &geometry);
};

#endif